/*
 * Copyright (c) 2004 Iwata <iwata@quasiquote.org>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <gauche/extend.h>
#include <gauche/uvector.h>
#include <ffi.h>

typedef struct ScmLazyRec {
	SCM_HEADER;
	ScmObj name;
	void *handle;
} ScmLazy;

SCM_CLASS_DECL(Scm_LazyClass);

#define SCM_CLASS_LAZY      (&Scm_LazyClass)
#define SCM_LAZY_FFI(obj)   ((ScmLazy*)obj)
#define SCM_LAZY_FFI_P(obj) SCM_XTYPEP(obj, SCM_CLASS_LAZY)

#define LAZY_VOID_TYPE       0
#define LAZY_INT_TYPE        1
#define LAZY_DOUBLE_TYPE     2
#define LAZY_PTR_TYPE        3
#define LAZY_CHAR_TYPE       4
#define LAZY_SCHEME_TYPE     5
#define LAZY_SCHEME_PTR_TYPE 6
#define LAZY_STRING_TYPE     7

extern char lazy_ffi_strerr[1024];

typedef struct ScmCpointerRec {
	SCM_HEADER;
	void *ptr;
} ScmCpointer;

SCM_CLASS_DECL(Scm_CpointerClass);

#define SCM_CLASS_CPOINTER  (&Scm_CpointerClass)
#define SCM_C_POINTER(obj)   ((ScmCpointer*)obj)
#define SCM_C_POINTER_P(obj) SCM_XTYPEP(obj, SCM_CLASS_CPOINTER)


/*
  prototypes
*/
extern ScmObj Scm_LazyFfiP();
extern ScmObj Scm_LazyStrerr();
extern ScmObj Scm_LazyOpen(ScmString *);
extern ScmObj Scm_LazyClose(ScmLazy *);
extern ScmObj Scm_LazyFunctionToCpointer(ScmLazy *, ScmString *);
extern ScmObj Scm_LazyCall(ScmLazy *, int, ScmString *, ScmObj, ScmObj);
extern ScmObj Scm_LazyCallWithCPointer(ScmLazy *, int, ScmCpointer *, ScmObj, ScmObj);

extern void finalize_lazy(GC_PTR, GC_PTR);

extern ScmObj Scm_LazyPointerP(ScmObj);
extern ScmObj Scm_LazyPointerAddress(ScmCpointer *);
extern ScmObj Scm_LazyMakeNullCpointer();
extern ScmObj Scm_LazyMakeCpointer(int);
extern ScmObj Scm_LazyNullCpointerP(ScmCpointer *);
extern ScmObj Scm_LazyAllocateForeignObject(int);
extern ScmObj Scm_LazyAllocateForeignString(int);
extern ScmObj Scm_LazyFreeForeignObject(ScmCpointer *);
extern ScmObj Scm_LazyConvertFromForeignString(ScmCpointer *);
extern ScmObj Scm_LazyConvertToForeignString(ScmString *);


extern ScmObj Scm_LazyCpointerToS8vector(int,  ScmCpointer *);
extern ScmObj Scm_LazyCpointerToU8vector(int,  ScmCpointer *);
extern ScmObj Scm_LazyCpointerToS16vector(int, ScmCpointer *);
extern ScmObj Scm_LazyCpointerToU16vector(int, ScmCpointer *);
extern ScmObj Scm_LazyCpointerToS32vector(int, ScmCpointer *);
extern ScmObj Scm_LazyCpointerToU32vector(int, ScmCpointer *);
extern ScmObj Scm_LazyCpointerToS64vector(int, ScmCpointer *);
extern ScmObj Scm_LazyCpointerToU64vector(int, ScmCpointer *);

extern ScmObj Scm_LazyS8vectorToCpointer(ScmS8Vector *);
extern ScmObj Scm_LazyU8vectorToCpointer(ScmU8Vector *);
extern ScmObj Scm_LazyS16vectorToCpointer(ScmS16Vector *);
extern ScmObj Scm_LazyU16vectorToCpointer(ScmU16Vector *);
extern ScmObj Scm_LazyS32vectorToCpointer(ScmS32Vector *);
extern ScmObj Scm_LazyU32vectorToCpointer(ScmU32Vector *);
extern ScmObj Scm_LazyS64vectorToCpointer(ScmS64Vector *);
extern ScmObj Scm_LazyU64vectorToCpointer(ScmU64Vector *);
